function [A,cond_A,a_ks] = function_TransferFunction_FDDT(z,lambda,delta,N,M,av)
% function_TransferFunction_FDDT() calculates the DDT transfer function used
% in the Frequency domain DDT (F-DDT) method with (or without) averaging as
% it is represented in [1]. It is assumed that :
% - the Fresnel approximation of the Rayleigh-Sommerfeld integral is used;
% - the pixel size at the sensor and object planes are equal;
% - the calculation is performed with a zero padded image to a double size
% (see the following example).
% Notation:
% [A,cond_A] = function_TransferFunctionFDDT(z,lambda,delta,N,M,av)
% A         the output F-DDT transfer function of the size N x M
% cond_A = max(|A|)/min(|A|) is the condition of the transfer function 
% The input setting variables:
% z         a distance between the input and output planes, [m]
% lambda    a wavelength, [m]
% delta     a pixel size, the same as for the Fourier image of a wave
%           field distribution (e.g. u0 in the following example), [m]
% N x M     a size of the function (# of rows x # of columns), [# of pixels]
% av        averaging factor (1/0 - averaged/non-averaged)
% 
% Example:
% The accurate F-DDT means that we use a double size calculation
% Let u0 is an object wave field distribution of the size N x M = 512 x 512
% >> N = 512; M = 512;
% >> u0_ = zeros(2*N,2*M);
% >> u0_(N-N/2+1:N+N/2,M-M/2+1:M+M/2)=u0;
% >> A = function_TransferFunction_FDDT(0.044,6328e-10,7.4e-6,2*N,2*M);
% >> uz = ifft2(fft2(u0_).*A);
% References:
% [1]  V. Katkovnik, J. Astola, and K. Egiazarian, Appl. Opt. 47, 3481-3493 (2008).
% 2010 (c) Artem Migukin, Vladimir Katkovnik,  v. 1.1.24.9.10

t=sqrt(-1)*pi/z/lambda;                                                     % a common factor
kf = -sqrt(-1)*exp(sqrt(-1)*2*pi*z/lambda)/(z+eps)/lambda*delta*delta;      % a factor depending on z only (for the Fresnel approx.)
xint=-M/2:M/2-1;yint=-N/2:N/2-1;                                            % the calculation is performed for a single row
[X,Y]=meshgrid(xint,yint);                                                  % sampling grid
a_ks=zeros(N,M);
fx = zeros(1,length(xint));fy = zeros(1,length(yint));
for q=1:length(xint)                                                        % we calculate a row of all kernel values
    if av
        fx(q)=quad(@(x)kernel(x,t,xint(q),delta),-1,1);                    % averaged values via a numerical integration
    else
        fx(q)=exp(t*delta^2*xint(q)^2);                                     % non averaged values
    end
end
if N==M                                                                     % acceleration is the matrix is square
    smoothed=fx(X+M/2+1).*fx(Y+N/2+1);    
else
    for q=1:length(yint)
        if av
            fy(q)=quad(@(x)kernel(x,t,yint(q),delta),-1,1);
        else
            fy(q)=exp(t*delta^2*yint(q)^2);
        end
    end
    smoothed=fx(X+M/2+1).*fy(Y+N/2+1); 
end   
y1=kf.*smoothed;                                                            % combine all 3 factors of the Fresnel approx. 
a_ks(2:end,2:end)=y1(2:end,2:end);                                          % in order to have in the center a DC term
% the kernel is shifted in order to obtain the 0th frequency in the upper
% left corner (for the correct multiplication with a FFT image)
A = fft2(circshift(a_ks,-1*round([N,M]/2)));
if nargout>=2
    cond_A = max(max(abs(A)))/min(min(abs(A)));                                 % the conditioning for the transfer function
end

% instrumental inline functions for numerical integration 
function y = kernel(x,t,d,delta)
    y = (1-abs(x)).*exp(t*delta^2*(d+x).^2);